/**
 * Sample number: 7 (of 20)
 * Result number: 7
 * Function name: int pthread_sigmask (int how, const sigset_t *new_mask, sigset_t *old_mask) #undef pthread_sigmask
 * File: lib/pthread_sigmask.c
 * Start: 34
 */
int
pthread_sigmask (int how, const sigset_t *new_mask, sigset_t *old_mask)
#undef pthread_sigmask
{
#if HAVE_PTHREAD_SIGMASK
  int ret;

# if PTHREAD_SIGMASK_INEFFECTIVE
  sigset_t omask, omask_copy;
  sigset_t *old_mask_ptr = &omask;
  sigemptyset (&omask);
  /* Add a signal unlikely to be blocked, so that OMASK_COPY
     is unlikely to match the actual mask.  */
  sigaddset (&omask, SIGILL);
  memcpy (&omask_copy, &omask, sizeof omask);
# else
  sigset_t *old_mask_ptr = old_mask;
# endif

  ret = pthread_sigmask (how, new_mask, old_mask_ptr);

# if PTHREAD_SIGMASK_INEFFECTIVE
  if (ret == 0)
    {
      /* Detect whether pthread_sigmask is currently ineffective.
         Don't cache the information: libpthread.so could be dynamically
         loaded after the program started and after pthread_sigmask was
         called for the first time.  */
      if (memcmp (&omask_copy, &omask, sizeof omask) == 0
          && pthread_sigmask (1729, &omask_copy, NULL) == 0)
        {
          /* pthread_sigmask is currently ineffective.  The program is not
             linked to -lpthread.  So use sigprocmask instead.  */
          return (sigprocmask (how, new_mask, old_mask) < 0 ? errno : 0);
        }

      if (old_mask)
        memcpy (old_mask, &omask, sizeof omask);
    }
# endif
# if PTHREAD_SIGMASK_FAILS_WITH_ERRNO
  if (ret == -1)
    return errno;
# endif
# if PTHREAD_SIGMASK_UNBLOCK_BUG
  if (ret == 0
      && new_mask != NULL
      && (how == SIG_UNBLOCK || how == SIG_SETMASK))
    {
      /* Give the OS the opportunity to raise signals that were pending before
         the pthread_sigmask call and have now been unblocked.  */
      usleep (1);
    }
# endif
  return ret;
#else
  int ret = sigprocmask (how, new_mask, old_mask);
  return (ret < 0 ? errno : 0);
#endif
}

/** End sample number: 7 */

